
Map=Core.class(Sprite)
Map.LoadingImg=Texture.new("Hourglass.png")

function Map:getTileUrl(tx,ty,tz)
	return self.source.getUrl(tx,ty,tz)
end

function Map:getTileFormat()
	return self.source.filetype
end

function Map:getTileTag(tx,ty,tz)
return self.source.code.."_"..tostring(tz).."_"..tostring(tx).."_"..tostring(ty)
end

function Map:evtMouseDown(evt)
	if self:hitTestPoint(evt.x,evt.y) then
		self.evtMoving=true
		self.evtSX=evt.x
		self.evtSY=evt.y
	end
end

function Map:evtMouseMove(evt)
	if self.evtMoving then
		local cy=MapMercator.LatToY(self.clat,self.zoom)-evt.y+self.evtSY
		local cx=MapMercator.LonToX(self.clon,self.zoom)-evt.x+self.evtSX
		self.evtSX=evt.x
		self.evtSY=evt.y
		clat=MapMercator.YToLat(cy,self.zoom)
		clon=MapMercator.XToLon(cx,self.zoom)
		self:setMapPosition(clat,clon)
	end
end

function Map:setMapPosition(lat,lon,zoom)
	if (lat~=nil) then
		self.clat=lat
	end
	if (lon~=nil) then
		self.clon=lon
	end
	if zoom~=nil then
		self.zoom=zoom
	end
	self:update()
end

function Map:getMapPosition()
 return self.clat,self.clon,self.zoom
end

function Map:setTileSource(tilesrc)
 self.source=tilesrc
 self:updateAll()
end

function Map:LLToXY(lat,lon)
 local cy=MapMercator.LatToY(self.clat,self.zoom)
 local cx=MapMercator.LonToX(self.clon,self.zoom)
 local ly=MapMercator.LatToY(lat,self.zoom)
 local lx=MapMercator.LonToX(lon,self.zoom)
 local bx=lx-cx+self.width/2
 local by=ly-cy+self.height/2
 return bx,by
end

function Map:XYToLL(x,y)
 local cy=MapMercator.LatToY(self.clat,self.zoom)-self.height/2+y
 local cx=MapMercator.LonToX(self.clon,self.zoom)-self.width/2+x
 local la=MapMercator.YToLat(cy,self.zoom)
 local lo=MapMercator.XToLon(cx,self.zoom)
 return la,lo
end

function Map:evtMouseUp(evt)
	self.evtMoving=false
end

function Map:init(w,h)
	Map.tileSz=MapMercator.TILE_SIZE
	self.source=MapTileSource.Mapnik
	
self.evtMoving=false

 self.width=w
 self.height=h
 self.zoom=10
 self.clat=48
 self.clon=2
 
 self:addEventListener(Event.MOUSE_DOWN,Map.evtMouseDown,self)
 self:addEventListener(Event.MOUSE_UP,Map.evtMouseUp,self)
 self:addEventListener(Event.MOUSE_MOVE,Map.evtMouseMove,self)
 self:updateAll()
end

function Map:updateAll()
 if (self.tiles~=nil) then
	for k,v in pairs(self.tiles) do
		v:removeFromParent()
	end
 end
 self.tiles={}
 self:update()
end

function Map:setupTile(tile,px,py)
	local w=Map.tileSz
	local h=Map.tileSz
	local tx=0
	local ty=0
	if (px==nil) then
		px=tile.opx
	end
	if (py==nil) then
		py=tile.opy
	end
	tile.opx=px
	tile.opy=py
	tx=0
	ty=0
	if px<0 then
		tx=-px
		w=w+px
		px=0
	end
	if py<0 then
		ty=-py
		h=h+py
		py=0
	end
	if (px+w>self.width) then
		w=self.width-px
	end
	if (py+h>self.height) then
		h=self.height-py
	end
	if (w==Map.tileSz) and (h==Map.tileSz) then
		tile:setTexture(tile.tileImg)
	else
		local tr=TextureRegion.new(tile.tileImg,tx,ty,w,h)
		tile:setTextureRegion(tr)
	end
	tile:setPosition(px,py)
end

function Map:update()
 local cy=MapMercator.LatToY(self.clat,self.zoom)
 local cx=MapMercator.LonToX(self.clon,self.zoom)
 local bx=cx-self.width/2
 local by=cy-self.height/2
 local tbasex=math.floor(bx/Map.tileSz)
 local tbasey=math.floor(by/Map.tileSz)
 local tbx=tbasex*Map.tileSz-bx
 local tby=tbasey*Map.tileSz-by
 print (self.clat,self.clon,cx,cy,bx,by,tbasex,tbasey,tbx,tby)
 
 local ntiles={}
 for tx=0,((self.width-tbx)/Map.tileSz),1 do
	for ty=0,((self.height-tby)/Map.tileSz),1 do
	  local ttag=self:getTileTag(tx+tbasex,ty+tbasey,self.zoom)
	  local ctile=self.tiles[ttag]
	  if ctile==nil then
		local tile=self:getTileUrl(tx+tbasex,ty+tbasey,self.zoom)
		local ldr=UrlLoader.new(tile)
		local spr=Bitmap.new(self.LoadingImg)
		spr.tileTag=ttag
		spr.tileFmt=self:getTileFormat()
		ldr:addEventListener(Event.COMPLETE, function (tgt,evt)
			local fn="|T|tile"..tgt.tileTag..tgt.tileFmt
			local out = io.open(fn, "wb")
			out:write(evt.data)
			out:close()
			local txt=Texture.new(fn)
			tgt.tileImg=txt
			os.remove(fn)
			local p=tgt:getParent()
			if (p~=nil) then
				p:setupTile(tgt)
			end
		end,spr)
		ctile=spr
		ctile.tileImg=self.LoadingImg
	  end
	  ntiles[ttag]=ctile
	  ctile:removeFromParent()
	  self:setupTile(ctile,tx*Map.tileSz+tbx,ty*Map.tileSz+tby)
	  self:addChild(ctile)
	end
 end
 for k,v in pairs(self.tiles) do
	if (ntiles[k]==nil) then
		v:removeFromParent()
	end
 end
 self.tiles=ntiles
end
